@@ -9,9 +9,7 @@
 ## Licence:     This program is free software; you can redistribute it and/or
 ##              modify it under the same terms as Perl itself
 ##
-## This module is actualy XML::Parser::Lite (with some updates). It's here
-## just for convenience.
-##
+## This module is actualy XML::Parser::Lite. It's just here for convenience.
 ## See original code at CPAN for full source and POD.
 ##
 ## This module will be used when XML::Parser is not installed.
@@ -25,8 +23,6 @@
 #
 # $Id: Lite.pm,v 1.4 2001/10/15 21:25:05 paulk Exp $
 #
-# Changes: Graciliano M. P. <gm@virtuasites.com.br>
-#
 # ======================================================================
 
 package XML::Smart::Parser ;
@@ -34,40 +30,31 @@ package XML::Smart::Parser ;
 no warnings ;
 use strict;
 use vars qw($VERSION);
-$VERSION = 1.2 ;
-
-my(@parsed , @stack, $level) ;
-
- &compile();
+$VERSION = sprintf("%d.%s", map {s/_//g; $_} q$Name: release-0_55-public $ =~ /-(\d+)_([\d_]+)/);
 
 sub new { 
-  my $class = ($_[0] =~ /^[\w:]+$/) ? shift(@_) : __PACKAGE__ ;
-  my $this = bless {} , $class ;
+  my $self = shift;
+  my $class = ref($self) || $self;
+  return $self if ref $self;
 
-  my %args = @_ ;
-  $this->setHandlers(%args) ;
-  
-  $this->{NOENTITY} = 1 ;
-
-  return $this ;
+  $self = bless {} => $class;
+  my %parameters = @_;
+  $self->setHandlers(); # clear first 
+  $self->setHandlers(%{$parameters{Handlers} || {}});
+  return $self;
 }
 
 sub setHandlers {
-  my $this = shift ;
-  my %args = @_;
-    
-  $this->{Init}  = $args{Init} || sub{} ;
-  $this->{Start} = $args{Start} || sub{} ;
-  $this->{Char}  = $args{Char} || sub{} ;
-  $this->{End}   = $args{End} || sub{} ;
-  $this->{Final} = $args{Final} || sub{} ;
-  
-  return 1 ;
+  my $self = shift; 
+  no strict 'refs'; local $^W;
+  unless (@_) { foreach (qw(Start End Char Final Init)) { *$_ = sub {} } }
+  while (@_) { my($name => $func) = splice(@_, 0, 2); *$name = defined $func ? $func : sub {} }
+  return $self;
 }
 
 sub regexp {
-  my $patch = shift || '' ;
-  my $package = __PACKAGE__ ;
+  my $patch = shift || '';
+  my $package = __PACKAGE__;
 
   my $TextSE = "[^<]+";
   my $UntilHyphen = "[^-]*-";
@@ -87,20 +74,18 @@ sub regexp {
   my $PI_Tail = "\\?>|$S1$UntilQMs(?:[^>?]$UntilQMs)*>";
   my $DT_ItemSE = "<(?:!(?:--$Until2Hyphens>|[^-]$MarkupDeclCE)|\\?$Name(?:$PI_Tail))|%$Name;|$S";
   my $DocTypeCE = "$DT_IdentSE(?:$S)?(?:\\[(?:$DT_ItemSE)*](?:$S)?)?>?";
-  my $DeclCE = "--(?:$CommentCE)?|\\[CDATA\\[(?:($CDATA_CE)(?{${package}::char_CDATA(\$2)}))?|DOCTYPE(?:$DocTypeCE)?";
+  my $DeclCE = "--(?:$CommentCE)?|\\[CDATA\\[(?:$CDATA_CE)?|DOCTYPE(?:$DocTypeCE)?";
   my $PI_CE = "$Name(?:$PI_Tail)?";
 
-  my $EndTagCE = "($Name)(?{${package}::end(\$3)})(?:$S)?>";
+  my $EndTagCE = "($Name)(?{${package}::end(\$2)})(?:$S)?>";
   my $AttValSE = "\"([^<\"]*)\"|'([^<']*)'";
-  my $ElemTagCE = "($Name)(?:$S($Name)(?:$S)?=(?:$S)?(?:$AttValSE)(?{[\@{\$^R||[]},\$5=>defined\$6?\$6:\$7]}))*(?:$S)?(/)?>(?{${package}::start(\$4,\@{\$^R||[]})})(?{\${8} and ${package}::end(\$4)})";
+  my $ElemTagCE = "($Name)(?:$S($Name)(?:$S)?=(?:$S)?(?:$AttValSE)(?{[\@{\$^R||[]},\$4=>defined\$5?\$5:\$6]}))*(?:$S)?(/)?>(?{${package}::start(\$3,\@{\$^R||[]})})(?{\${7} and ${package}::end(\$3)})";
   my $MarkupSPE = "<(?:!(?:$DeclCE)?|\\?(?:$PI_CE)?|/(?:$EndTagCE)?|(?:$ElemTagCE)?)";
 
   "(?:($TextSE)(?{${package}::char(\$1)}))$patch|$MarkupSPE";
 }
 
-sub compile {
-  local $^W; 
-  
+sub compile { local $^W; 
   foreach (regexp(), regexp('??')) {
     eval qq{sub parse_re { use re "eval"; 1 while \$_[0] =~ m{$_}go }; 1} or die;
     last if eval { parse_re('<foo>bar</foo>'); 1 }
@@ -109,71 +94,49 @@ sub compile {
   *compile = sub {};
 }
 
-sub parse {
-  my $this = shift ;
-  
-  @parsed = () ;
-  
-  init();
-  parse_re($_[0]);
-  final();
+setHandlers();
+compile();
 
-  no strict qw(refs);
-  
-  my $final = pop(@parsed) ; pop(@parsed) ;
-
-  for (my $i = 0 ; $i <= $#parsed ; $i+=2) {
-    my $args = $parsed[$i+1] ;
-    &{$this->{$parsed[$i]}}($this , (ref($args) ? @{$args} : $args) ) ;
-  }
-
-  @parsed = () ;
-
-  return &{$this->{Final}}($this, @{$final}) ;
+sub parse { 
+  init(); 
+  parse_re($_[1]);
+  final(); 
 }
 
-sub init {
+my(@stack, $level);
+
+sub init { 
   @stack = (); $level = 0;
-  push(@parsed , 'Init' , [@_]) ;
-  return ;
+  Init(__PACKAGE__, @_);  
 }
 
-sub final {
+sub final { 
   die "not properly closed tag '$stack[-1]'\n" if @stack;
   die "no element found\n" unless $level;
-  push(@parsed , 'Final' , [@_]) ;
-  return ;
+  Final(__PACKAGE__, @_) 
 } 
 
-sub start {
+sub start { 
   die "multiple roots, wrong element '$_[0]'\n" if $level++ && !@stack;
   push(@stack, $_[0]);
-  push(@parsed , 'Start' , [@_]) ;
-  return ;
+  Start(__PACKAGE__, @_); 
 }
 
-sub char {
-  push(@parsed , 'Char' , [@_]) , return if @stack;
+sub char { 
+  Char(__PACKAGE__, $_[0]), return if @stack;
 
   for (my $i=0; $i < length $_[0]; $i++) {
     die "junk '$_[0]' @{[$level ? 'after' : 'before']} XML element\n"
       if index("\n\r\t ", substr($_[0],$i,1)) < 0; # or should '< $[' be there
   }
-  return ;
 }
 
-sub char_CDATA {
-  &char( substr($_[0] , 0 , -3) ) ;
-}
-
-sub end {
+sub end { 
   pop(@stack) eq $_[0] or die "mismatched tag '$_[0]'\n";
-  push(@parsed , 'End' , [@_]) ;
-  return ;
+  End(__PACKAGE__, $_[0]);
 }
 
 # ======================================================================
 
 1;
 
-
